# SPDX-FileCopyrightText: 2017 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_max7219.bcddigits.BCDDigits`
====================================================
"""
from micropython import const
from adafruit_max7219 import max7219

try:
    # Used only for typing
    from typing import List
    import digitalio
    import busio
except ImportError:
    pass

__version__ = "1.5.6"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MAX7219.git"

_DECODEMODE = const(9)
_SCANLIMIT = const(11)
_SHUTDOWN = const(12)
_DISPLAYTEST = const(15)


class BCDDigits(max7219.MAX7219):
    """
    Basic support for display on a 7-Segment BCD display controlled
    by a Max7219 chip using SPI.

    :param ~busio.SPI spi: an spi busio or spi bitbangio object
    :param ~digitalio.DigitalInOut cs: digital in/out to use as chip select signal
    :param int nDigits: number of led 7-segment digits; default 1; max 8
    """

    def __init__(self, spi: busio.SPI, cs: digitalio.DigitalInOut, nDigits: int = 1):
        self._ndigits = nDigits
        super().__init__(self._ndigits, 8, spi, cs)

    def init_display(self) -> None:

        for cmd, data in (
            (_SHUTDOWN, 0),
            (_DISPLAYTEST, 0),
            (_SCANLIMIT, 7),
            (_DECODEMODE, (2**self._ndigits) - 1),
            (_SHUTDOWN, 1),
        ):
            self.write_cmd(cmd, data)

        self.clear_all()
        self.show()

    def set_digit(self, dpos: int, value: int) -> None:
        """
        Display one digit.

        :param int dpos: the digit position; zero-based
        :param int value: integer ranging from 0->15
        """
        dpos = self._ndigits - dpos - 1
        for i in range(4):
            # print('digit {} pixel {} value {}'.format(dpos,i+4,v & 0x01))
            self.pixel(dpos, i, value & 0x01)
            value >>= 1

    def set_digits(self, start: int, values: List[int]) -> None:
        """
        Display digits from a list.

        :param int start: digit to start display zero-based
        :param list[int] values: list of integer values ranging from 0->15
        """
        for value in values:
            # print('set digit {} start {}'.format(d,start))
            self.set_digit(start, value)
            start += 1

    def show_dot(self, dpos: int, bit_value: int = None) -> None:
        """
        The decimal point for a digit.

        :param int dpos: the digit to set the decimal point zero-based
        :param int bit_value: value > zero lights the decimal point, else unlights the point
        """
        if 0 <= dpos < self._ndigits:
            # print('set dot {} = {}'.format((self._ndigits - d -1),col))
            self.pixel(self._ndigits - dpos - 1, 7, bit_value)

    def clear_all(self) -> None:
        """
        Clear all digits and decimal points.
        """
        self.fill(1)
        for i in range(self._ndigits):
            self.show_dot(i)

    def show_str(self, start: int, strg: str) -> None:
        """
        Displays a numeric str in the display.  Shows digits ``0-9``, ``-``, and ``.``.

        :param int start: start position to show the numeric string
        :param str strg: the numeric string
        """
        cpos = start
        for char in strg:
            # print('c {}'.format(c))
            value = 0x0F  # assume blank
            if "0" <= char <= "9":
                value = int(char)
            elif char == "-":
                value = 10  # minus sign
            elif char == ".":
                self.show_dot(cpos - 1, 1)
                continue
            self.set_digit(cpos, value)
            cpos += 1

    def show_help(self, start: int) -> None:
        """
        Display the word HELP in the display.

        :param int start: start position to show HELP
        """
        digits = [12, 11, 13, 14]
        self.set_digits(start, digits)



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

import time
import board
import digitalio
from adafruit_max7219 import matrices


# You may need to change the chip select pin depending on your wiring
spi = board.SPI()
cs = digitalio.DigitalInOut(board.D4)

matrix = matrices.Matrix8x8(spi, cs)
while True:
    print("Cycle start")
    # all lit up
    matrix.fill(True)
    matrix.show()
    time.sleep(0.5)

    # all off
    matrix.fill(False)
    matrix.show()
    time.sleep(0.5)

    # one column of leds lit
    for i in range(8):
        matrix.pixel(1, i, 1)
    matrix.show()
    time.sleep(0.5)
    # now scroll the column to the right
    for j in range(8):
        matrix.scroll(1, 0)
        matrix.show()
        time.sleep(0.5)

    # show a string one character at a time
    adafruit = "Adafruit"
    for char in adafruit:
        matrix.fill(0)
        matrix.text(char, 0, 0)
        matrix.show()
        time.sleep(1.0)

    # scroll the last character off the display
    for i in range(8):
        matrix.scroll(-1, 0)
        matrix.show()
        time.sleep(0.5)

    # scroll a string across the display
    for pixel_position in range(len(adafruit) * 8):
        matrix.fill(0)
        matrix.text(adafruit, -pixel_position, 0)
        matrix.show()
        time.sleep(0.25)



# SPDX-FileCopyrightText: 2021 Daniel Flanagan
# SPDX-License-Identifier: MIT

import time
import board
import digitalio
from adafruit_max7219 import matrices


# You may need to change the chip select pin depending on your wiring
spi = board.SPI()
cs = digitalio.DigitalInOut(board.D4)

matrix = matrices.CustomMatrix(spi, cs, 32, 8)
while True:
    print("Cycle Start")
    # all lit up
    matrix.fill(True)
    matrix.show()
    time.sleep(0.5)

    # all off
    matrix.fill(False)
    matrix.show()
    time.sleep(0.5)

    # snake across panel
    for y in range(8):
        for x in range(32):
            if not y % 2:
                matrix.pixel(x, y, 1)
            else:
                matrix.pixel(31 - x, y, 1)
            matrix.show()
            time.sleep(0.05)

    # show a string one character at a time
    adafruit = "Adafruit"
    matrix.fill(0)
    for i, char in enumerate(adafruit[:3]):
        matrix.text(char, i * 6, 0)
        matrix.show()
        time.sleep(1.0)
    matrix.fill(0)
    for i, char in enumerate(adafruit[3:]):
        matrix.text(char, i * 6, 0)
        matrix.show()
        time.sleep(1.0)

    # scroll the last character off the display
    for i in range(32):
        matrix.scroll(-1, 0)
        matrix.show()
        time.sleep(0.25)

    # scroll a string across the display
    for pixel_position in range(len(adafruit) * 8):
        matrix.fill(0)
        matrix.text(adafruit, -pixel_position, 0)
        matrix.show()
        time.sleep(0.25)



# SPDX-FileCopyrightText: 2016 Philip R. Moyer  for Adafruit Industries
# SPDX-FileCopyrightText: 2016 Radomir Dopieralski for Adafruit Industries
# SPDX-FileCopyrightText: 2021 Daniel Flanagan
#
# SPDX-License-Identifier: MIT

"""
`adafruit_max7219.max7219` - MAX7219 LED Matrix/Digit Display Driver
========================================================================
CircuitPython library to support MAX7219 LED Matrix/Digit Display Driver.
This library supports the use of the MAX7219-based display in CircuitPython,
either an 8x8 matrix or a 8 digit 7-segment numeric display.

See Also
=========
* matrices.Maxtrix8x8 is a class support an 8x8 led matrix display
* matrices.CustomMatrix is a class support a custom sized constellation of 8x8 led matrix displays
* bcddigits.BCDDigits is a class that support the 8 digit 7-segment display

Beware that most CircuitPython compatible hardware are 3.3v logic level! Make
sure that the input pin is 5v tolerant.

* Author(s): Michael McWethy

Implementation Notes
--------------------
**Hardware:**

* Adafruit `MAX7219CNG LED Matrix/Digit Display Driver -
  MAX7219 <https://www.adafruit.com/product/453>`_ (Product ID: 453)

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the ESP8622 and M0-based boards:
  https://github.com/adafruit/circuitpython/releases

* Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice

**Notes:**
#.  Datasheet: https://cdn-shop.adafruit.com/datasheets/MAX7219.pdf
"""
# MicroPython MAX7219 driver, SPI interfaces
import digitalio
from adafruit_bus_device import spi_device
from micropython import const
import adafruit_framebuf as framebuf

try:
    # Used only for typing
    import typing  # pylint: disable=unused-import
    import busio
except ImportError:
    pass

__version__ = "1.5.6"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MAX7219.git"

# register definitions
_DIGIT0 = const(1)
_INTENSITY = const(10)


class MAX7219:
    """
    MAX7219 - driver for displays based on max7219 chip_select

    :param int width: the number of pixels wide
    :param int height: the number of pixels high
    :param ~busio.SPI spi: an spi busio or spi bitbangio object
    :param ~digitalio.DigitalInOut chip_select: digital in/out to use as chip select signal
    :param int baudrate: for SPIDevice baudrate (default 8000000)
    :param int polarity: for SPIDevice polarity (default 0)
    :param int phase: for SPIDevice phase (default 0)
    """

    def __init__(
        self,
        width: int,
        height: int,
        spi: busio.SPI,
        cs: digitalio.DigitalInOut,
        *,
        baudrate: int = 8000000,
        polarity: int = 0,
        phase: int = 0
    ):

        self._chip_select = cs
        self._chip_select.direction = digitalio.Direction.OUTPUT

        self._spi_device = spi_device.SPIDevice(
            spi, cs, baudrate=baudrate, polarity=polarity, phase=phase
        )

        self._buffer = bytearray((height // 8) * width)
        self.framebuf = framebuf.FrameBuffer1(self._buffer, width, height)

        self.width = width
        self.height = height

        self.init_display()

    def init_display(self) -> None:
        """Must be implemented by derived class (``matrices``, ``bcddigits``)"""

    def brightness(self, value: int) -> None:
        """
        Controls the brightness of the display.

        :param int value: 0->15 dimmest to brightest
        """
        if not 0 <= value <= 15:
            raise ValueError("Brightness out of range")
        self.write_cmd(_INTENSITY, value)

    def show(self) -> None:
        """
        Updates the display.
        """
        for ypos in range(8):
            self.write_cmd(_DIGIT0 + ypos, self._buffer[ypos])

    def fill(self, bit_value: int) -> None:
        """
        Fill the display buffer.

        :param int bit_value: value > 0 set the buffer bit, else clears the buffer bit
        """
        self.framebuf.fill(bit_value)

    def pixel(self, xpos: int, ypos: int, bit_value: int = None) -> None:
        """
        Set one buffer bit

        :param int xpos: x position to set bit
        :param int ypos: y position to set bit
        :param int bit_value: value > 0 sets the buffer bit, else clears the buffer bit
        """
        bit_value = 0x01 if bit_value else 0x00
        self.framebuf.pixel(xpos, ypos, bit_value)

    def scroll(self, delta_x: int, delta_y: int) -> None:
        """
        Srcolls the display using delta_x,delta_y.

        :param int delta_x: positions to scroll in the x direction
        :param int delta_y: positions to scroll in the y direction
        """
        self.framebuf.scroll(delta_x, delta_y)

    def write_cmd(self, cmd: int, data: int) -> None:
        """
        Writes a command to spi device.

        :param int cmd: register address to write data to
        :param int data: data to be written to commanded register
        """
        # print('cmd {} data {}'.format(cmd,data))
        self._chip_select.value = False
        with self._spi_device as my_spi_device:
            my_spi_device.write(bytearray([cmd, data]))


class ChainableMAX7219(MAX7219):
    """
    Daisy Chainable MAX7219 - driver for cascading displays based on max7219 chip_select

    :param int width: the number of pixels wide
    :param int height: the number of pixels high
    :param ~busio.SPI spi: an spi busio or spi bitbangio object
    :param ~digitalio.DigitalInOut chip_select: digital in/out to use as chip select signal
    :param int baudrate: for SPIDevice baudrate (default 8000000)
    :param int polarity: for SPIDevice polarity (default 0)
    :param int phase: for SPIDevice phase (default 0)
    """

    def __init__(
        self,
        width: int,
        height: int,
        spi: busio.SPI,
        cs: digitalio.DigitalInOut,
        *,
        baudrate: int = 8000000,
        polarity: int = 0,
        phase: int = 0
    ):
        self.chain_length = (height // 8) * (width // 8)

        super().__init__(
            width, height, spi, cs, baudrate=baudrate, polarity=polarity, phase=phase
        )
        self._buffer = bytearray(self.chain_length * 8)
        self.framebuf = framebuf.FrameBuffer1(self._buffer, self.chain_length * 8, 8)

    def write_cmd(self, cmd: int, data: int) -> None:
        """
        Writes a command to spi device.

        :param int cmd: register address to write data to
        :param int data: data to be written to commanded register
        """
        # print('cmd {} data {}'.format(cmd,data))
        self._chip_select.value = False
        with self._spi_device as my_spi_device:
            for _ in range(self.chain_length):
                my_spi_device.write(bytearray([cmd, data]))

    def show(self) -> None:
        """
        Updates the display.
        """
        for ypos in range(8):
            self._chip_select.value = False
            with self._spi_device as my_spi_device:
                for chip in range(self.chain_length):
                    my_spi_device.write(
                        bytearray(
                            [
                                _DIGIT0 + ypos,
                                self._buffer[ypos * self.chain_length + chip],
                            ]
                        )
                    )



# SPDX-FileCopyrightText: 2017 Dan Halbert for Adafruit Industries
# SPDX-FileCopyrightText: 2021 Daniel Flanagan
#
# SPDX-License-Identifier: MIT

"""
`adafruit_max7219.matrices`
====================================================
"""
from micropython import const
from adafruit_framebuf import BitmapFont
from adafruit_max7219 import max7219

try:
    # Used only for typing
    from typing import Tuple
    import digitalio
    import busio
except ImportError:
    pass

__version__ = "1.5.6"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MAX7219.git"

_DECODEMODE = const(9)
_SCANLIMIT = const(11)
_SHUTDOWN = const(12)
_DISPLAYTEST = const(15)


class Matrix8x8(max7219.MAX7219):
    """
    Driver for a 8x8 LED matrix based on the MAX7219 chip.

    :param ~busio.SPI spi: an spi busio or spi bitbangio object
    :param ~digitalio.DigitalInOut cs: digital in/out to use as chip select signal
    """

    def __init__(self, spi: busio.SPI, cs: digitalio.DigitalInOut):
        super().__init__(8, 8, spi, cs)

    def init_display(self) -> None:
        for cmd, data in (
            (_SHUTDOWN, 0),
            (_DISPLAYTEST, 0),
            (_SCANLIMIT, 7),
            (_DECODEMODE, 0),
            (_SHUTDOWN, 1),
        ):
            self.write_cmd(cmd, data)

        self.fill(0)
        self.show()

    def text(self, strg: str, xpos: int, ypos: int, bit_value: int = 1) -> None:
        """
        Draw text in the 8x8 matrix.

        :param str strg: string to place in to display
        :param int xpos: x position of LED in matrix
        :param int ypos: y position of LED in matrix
        :param int bit_value: > 1 sets the text, otherwise resets
        """
        self.framebuf.text(strg, xpos, ypos, bit_value)

    def clear_all(self) -> None:
        """
        Clears all matrix leds.
        """
        self.fill(0)


class CustomMatrix(max7219.ChainableMAX7219):
    """
    Driver for a custom 8x8 LED matrix constellation based on daisy chained MAX7219 chips.

    :param ~busio.SPI spi: an spi busio or spi bitbangio object
    :param ~digitalio.DigitalInOut cs: digital in/out to use as chip select signal
    :param int width: the number of pixels wide
    :param int height: the number of pixels high
    :param int rotation: the number of times to rotate the coordinate system (default 1)
    """

    def __init__(
        self,
        spi: busio.SPI,
        cs: digitalio.DigitalInOut,
        width: int,
        height: int,
        *,
        rotation: int = 1
    ):
        super().__init__(width, height, spi, cs)

        self.y_offset = width // 8
        self.y_index = self._calculate_y_coordinate_offsets()

        self.framebuf.rotation = rotation
        self.framebuf.fill_rect = self._fill_rect
        self._font = None

    def _calculate_y_coordinate_offsets(self) -> None:
        y_chunks = []
        for _ in range(self.chain_length // (self.width // 8)):
            y_chunks.append([])
        chunk = 0
        chunk_size = 0
        for index in range(self.chain_length * 8):
            y_chunks[chunk].append(index)
            chunk_size += 1
            if chunk_size >= (self.width // 8):
                chunk_size = 0
                chunk += 1
                if chunk >= len(y_chunks):
                    chunk = 0

        y_index = []
        for chunk in y_chunks:
            y_index += chunk
        return y_index

    def init_display(self) -> None:
        for cmd, data in (
            (_SHUTDOWN, 0),
            (_DISPLAYTEST, 0),
            (_SCANLIMIT, 7),
            (_DECODEMODE, 0),
            (_SHUTDOWN, 1),
        ):
            self.write_cmd(cmd, data)

        self.fill(0)
        self.show()

    def clear_all(self) -> None:
        """
        Clears all matrix leds.
        """
        self.fill(0)

    # pylint: disable=inconsistent-return-statements
    def pixel(self, xpos: int, ypos: int, bit_value: int = None) -> None:
        """
        Set one buffer bit

        :param int xpos: x position to set bit
        :param int ypos: y position to set bit
        :param int bit_value: value > 0 sets the buffer bit, else clears the buffer bit
        """
        if xpos < 0 or ypos < 0 or xpos >= self.width or ypos >= self.height:
            return
        buffer_x, buffer_y = self._pixel_coords_to_framebuf_coords(xpos, ypos)
        return super().pixel(buffer_x, buffer_y, bit_value=bit_value)

    def _pixel_coords_to_framebuf_coords(self, xpos: int, ypos: int) -> Tuple[int]:
        """
        Convert matrix pixel coordinates into coordinates in the framebuffer

        :param int xpos: x position
        :param int ypos: y position
        :return: framebuffer coordinates (x, y)
        :rtype: Tuple[int]
        """
        return (xpos - ((xpos // 8) * 8)) % 8, xpos // 8 + self.y_index[
            ypos * self.y_offset
        ]

    def _get_pixel(self, xpos: int, ypos: int) -> int:
        """
        Get value of a matrix pixel

        :param int xpos: x position
        :param int ypos: y position
        :return: value of pixel in matrix
        :rtype: int
        """
        x, y = self._pixel_coords_to_framebuf_coords(xpos, ypos)
        buffer_value = self._buffer[-1 * y - 1]
        return ((buffer_value & 2**x) >> x) & 1

    # Adafruit Circuit Python Framebuf Scroll Function
    # Authors: Kattni Rembor, Melissa LeBlanc-Williams and Tony DiCola, for Adafruit Industries
    # License: MIT License (https://opensource.org/licenses/MIT)
    def scroll(self, delta_x: int, delta_y: int) -> None:
        """
        Srcolls the display using delta_x, delta_y.

        :param int delta_x: positions to scroll in the x direction
        :param int delta_y: positions to scroll in the y direction
        """
        if delta_x < 0:
            shift_x = 0
            xend = self.width + delta_x
            dt_x = 1
        else:
            shift_x = self.width - 1
            xend = delta_x - 1
            dt_x = -1
        if delta_y < 0:
            y = 0
            yend = self.height + delta_y
            dt_y = 1
        else:
            y = self.height - 1
            yend = delta_y - 1
            dt_y = -1
        while y != yend:
            x = shift_x
            while x != xend:
                self.pixel(x, y, self._get_pixel(x - delta_x, y - delta_y))
                x += dt_x
            y += dt_y

    def rect(
        self, x: int, y: int, width: int, height: int, color: int, fill: bool = False
    ) -> None:
        """
        Draw a rectangle at the given position of the given size, color, and fill.

        :param int x: x position
        :param int y: y position
        :param int width: width of rectangle
        :param int height: height of rectangle
        :param int color: color of rectangle
        :param bool fill: 1 pixel outline or filled rectangle (default: False)
        """
        # pylint: disable=too-many-arguments
        for row in range(height):
            y_pos = row + y
            for col in range(width):
                x_pos = col + x
                if fill:
                    self.pixel(x_pos, y_pos, color)
                elif y_pos in (y, y + height - 1) or x_pos in (x, x + width - 1):
                    self.pixel(x_pos, y_pos, color)
                else:
                    continue

    def _fill_rect(self, x: int, y: int, width: int, height: int, color: int) -> None:
        """
        Draw a filled rectangle at the given position of the given size, color.

        :param int x: x position
        :param int y: y position
        :param int width: width of rectangle
        :param int height: height of rectangle
        :param int color: color of rectangle
        """
        # pylint: disable=too-many-arguments
        return self.rect(x, y, width, height, color, True)

    # Adafruit Circuit Python Framebuf Text Function
    # Authors: Kattni Rembor, Melissa LeBlanc-Williams and Tony DiCola, for Adafruit Industries
    # License: MIT License (https://opensource.org/licenses/MIT)
    def text(
        self,
        strg: str,
        xpos: int,
        ypos: int,
        color: int = 1,
        *,
        font_name: str = "font5x8.bin",
        size: int = 1
    ) -> None:
        """
        Draw text in the matrix.

        :param str strg: string to place in to display
        :param int xpos: x position of LED in matrix
        :param int ypos: y position of LED in matrix
        :param int color: > 1 sets the text, otherwise resets
        :param str font_name: path to binary font file (default: "font5x8.bin")
        :param int size: size of the font, acts as a multiplier
        """
        for chunk in strg.split("\n"):
            if not self._font or self._font.font_name != font_name:
                # load the font!
                self._font = BitmapFont(font_name)
            width = self._font.font_width
            height = self._font.font_height
            for i, char in enumerate(chunk):
                char_x = xpos + (i * (width + 1)) * size
                if (
                    char_x + (width * size) > 0
                    and char_x < self.width
                    and ypos + (height * size) > 0
                    and ypos < self.height
                ):
                    self._font.draw_char(
                        char, char_x, ypos, self.framebuf, color, size=size
                    )
            ypos += height * size



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

import time
import random
import board
import digitalio
from adafruit_max7219 import bcddigits


# You may need to change the chip select pin depending on your wiring
spi = board.SPI()
cs = digitalio.DigitalInOut(board.D4)

leds = bcddigits.BCDDigits(spi, cs, nDigits=8)
while True:
    # clear display and dim 0
    leds.brightness(0)
    leds.clear_all()

    # place 8-digit number on display
    value = 12345678
    leds.show_str(0, "{:8}".format(value))
    leds.show()

    # increase the brightness slowly
    for i in range(16):
        leds.brightness(i)
        time.sleep(0.5)

    leds.brightness(3)

    # show "-HELP-90" on display
    leds.show_str(6, "90")  # show 90 starting at position 6
    leds.set_digit(0, 10)  # show - at position 0
    leds.set_digit(1, 12)  # show H at position 1
    leds.set_digit(2, 11)  # show E at position 2
    leds.set_digit(3, 13)  # show L at position 3
    leds.set_digit(4, 14)  # show P at position 4
    leds.set_digit(5, 10)  # show - at position 5

    leds.show()
    time.sleep(1.0)

    leds.clear_all()
    leds.brightness(5)

    # set the two dots and two 4-digit numbers
    leds.show_dot(2, 1)
    leds.show_dot(6, 1)
    leds.show_str(0, " 72.5")
    leds.show_str(4, "-10.8")

    leds.show()
    time.sleep(1.0)

    leds.brightness(10)
    leds.clear_all()
    # show a 4 character numeric string
    leds.show_str(0, "   0")
    leds.show()
    time.sleep(1.0)

    leds.clear_all()
    # show 0->8
    for digit in range(8):
        leds.set_digit(digit, digit)

    leds.show()
    time.sleep(1.0)

    # show random 8-digit numbers via show_str
    for _ in range(10):
        number = random.uniform(-1.0, 1.0)
        number *= 10000.0
        number_string = "{:9.3f}".format(number)
        leds.clear_all()
        leds.show_str(0, number_string)
        leds.show()
        time.sleep(1.0)

    # show the help string
    leds.clear_all()
    leds.show_help(2)
    leds.show()

    time.sleep(1.0)
